/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qstringlist.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qframe.h>

#include <klocale.h>
#include <kconfig.h>
#include <kfiletreeview.h>
#include <kfileiconview.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klistview.h>
#include <kaction.h>
#include <kmainwindow.h>
#include <kstatusbar.h>
#include <kprogress.h>
#include <kcmdlineargs.h>
#include <kdiroperator.h>
#include <kfiledialog.h>

#include "kdigestwidget.h"
#include "digestfile.h"
#include "sfvfile.h"
#include "md5sumfile.h"

kdigestWidget::kdigestWidget(QWidget* parent, const char* name, WFlags fl)
//        : kdigestWidgetBase(parent,name,fl)
        : QWidget(parent,name,fl)
{
	QHBoxLayout *wl = new QHBoxLayout(this, 11, 6);

	m_splitter = new QSplitter(this, "splitter");
	m_splitter->setOpaqueResize(true);
	
	QLayout *sl = new QGridLayout(m_splitter);
	
	m_allDirs = new KFileTreeView(m_splitter, "allDirs");
        m_allDirs->addColumn( tr( "Directory" ) );
	m_allDirs->setMaximumSize(200, 32767);
        m_allDirs->setResizePolicy( QListView::AutoOneFit );
        m_allDirs->setRootIsDecorated( true );
	sl->add(m_allDirs);
	connect( m_allDirs, SIGNAL( selectionChanged(QListViewItem*) ), this, SLOT( dirSelected() ) );
	
	m_fileFrame = new QFrame( m_splitter, "fileFrame" );
	m_fileFrame->setGeometry( QRect( 260, 10, 352, 370 ) );
	m_fileFrame->setFrameShape( QFrame::NoFrame );
	m_fileFrame->setFrameShadow( QFrame::Plain );
	m_fileFrame->setLineWidth( 0 );
	m_fileFrame->setMaximumSize(32767, 32767);
	QLayout *fileFrameLayout = new QVBoxLayout( m_fileFrame, 0, 6, "fileFrameLayout");

	///@todo move it to the home instead of root
	m_currDir = new KDirOperator( KURL("file:///"), m_fileFrame, "currDir" );
	m_currDir->setMinimumSize( QSize( 350, 150 ) );
	m_currDir->setMaximumSize( QSize(32767, 32767) );
	m_currDir->setView( KFile::Simple );
	m_currDir->setMode( KFile::Files );
	///@todo Should be KFile::Directory also, but until the navigation is not complete, omit it
	m_currDir->readConfig( KGlobal::config(), "currentDir" );
	fileFrameLayout->add( m_currDir );
	connect( m_currDir, SIGNAL( fileSelected(const KFileItem*) ), this, SLOT( fileHighlighted(const KFileItem*) ) );
	connect( m_currDir, SIGNAL( dirActivated(const KFileItem*) ), this, SLOT( dirActivated(const KFileItem*) ) );

	m_content = new KListView( m_fileFrame, "m_content" );
	m_content->addColumn( tr2i18n( "Filename" ) );
	m_content->addColumn( tr2i18n( "Registered CRC" ) );
	m_content->addColumn( tr2i18n( "Actual CRC" ) );
	fileFrameLayout->add( m_content );
	m_content->setMaximumSize(32767, 32767);
	
	m_splitter->setCollapsible(m_allDirs, false);
	m_splitter->setCollapsible(m_fileFrame, false);
	wl->add(m_splitter);
	
	KFileTreeBranch *home = m_allDirs->addBranch(
		QDir::home().path(),
		"Home",
		KGlobal::iconLoader()->loadIcon("folder_home", KIcon::Small)
	);
	m_allDirs->setDirOnlyMode(home, true);
	m_root = m_allDirs->addBranch(
		"file:/",
		"Root",
		KGlobal::iconLoader()->loadIcon("folder_red", KIcon::Small)
	);
	m_allDirs->setDirOnlyMode(m_root, true);
	
	m_aCheck = new KAction(
		i18n("&Check all files"),	// Shown
		"run",				// icon
		0,				// shortcut
		this,
		SLOT( checkFile() ),
		(reinterpret_cast<KMainWindow*>(parent))->actionCollection(),
		"checkfile"
		);
	m_aCheck->setEnabled(false);
	
	m_aConvertAFV = new KAction(
		i18n("Convert &AFV file to SFV"),
		NULL,
		0,
		this,
		SLOT( convertAFV() ),
		(reinterpret_cast<KMainWindow*>(parent))->actionCollection(),
		"convertafv"
		);
	m_aConvertAFV->setEnabled(false);
	
	m_aCreate = new KAction(
		i18n("Create a digest file"),
		NULL,
		0,
		this,
		SLOT( createDigest() ),
		(reinterpret_cast<KMainWindow*>(parent))->actionCollection(),
		"createdigest"
		);
	
	m_statusBar = (reinterpret_cast<KMainWindow*>(parent))->statusBar();

	m_statusBar->setSizePolicy( QSizePolicy( QSizePolicy::Ignored, QSizePolicy::Fixed ) );
	m_statusBar->setMinimumHeight(25);
	m_statusBar->setMaximumHeight(25);
	
	m_statusLabel = new QLabel(this);
	m_statusLabel->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
	m_statusLabel->setMinimumHeight(25);
	m_statusLabel->setMaximumHeight(25);
	m_statusBar->addWidget(m_statusLabel);
	
	m_progress = new KProgress(this);
	m_progress->setEnabled(false);
	m_progress->setMaximumSize( QSize(75, 20) );
	m_statusBar->addWidget(m_progress, 0, true);
	
	m_allDirs->repaint();

	m_file = NULL;
	
//	if ( KCmdLineArgs::parsedArgs()->isSet("filename") )
	{
		///@todo Open the file requested
	}
}

kdigestWidget::~kdigestWidget()
{
	delete m_file;
}

/**
@brief Changes the directory shown up on m_currDir and update m_allDirs
@param newpath Path to move the interface to
*/
inline void kdigestWidget::changeDir(const KURL &newpath)
{
	m_content->clear();
	m_currDir->setURL( newpath, false );
}

/**
@brief Slot called when the m_allDirs selection is changed
@param lvi KFileListViewItem selected
*/
void kdigestWidget::dirSelected()
{
	changeDir(m_allDirs->currentURL());
}

/**
@brief Slot called when is highlighted a file into m_currDir
@param fi File highlighted
*/
void kdigestWidget::fileHighlighted(const KFileItem* fi)
{
	if ( ! fi ) return;
	cleanup();
	
	m_aConvertAFV->setEnabled(false);
	
	if (
		fi->name().right(4).lower() == ".sfv" ||
		fi->name().right(4).lower() == ".afv"    // Let's consider afv as an exception of sfv
	) {
		m_file = new KDigest::SFVfile(fi->url().path(), this, "sfvfile");
		rereadEntries();
		
		if ( fi->name().right(4).lower() == ".afv" )
			m_aConvertAFV->setEnabled(true);
	} else if (
		fi->name() == "MD5SUM" ||
		fi->name() == "MD5SUMS" ||
		fi->name().right(4).lower() == ".md5"
	) {
		m_file = new KDigest::MD5SUMfile(fi->url().path(), this, "md5sumfile");
		rereadEntries();
	}
	
	// Now update the actions....
	m_aCheck->setEnabled( m_file );
}

/**
@brief Slot called when a directory in m_currDir is activated
@param fi directory activated
*/
void kdigestWidget::dirActivated(const KFileItem* fi)
{
	delete m_file;
	m_file = NULL;
	m_content->clear();
	m_aConvertAFV->setEnabled(false);
	m_aCheck->setEnabled(false);
	
	QDir f( fi->url().path() ); f.cd("..");
	
	KFileTreeViewItem *tvi = m_allDirs->findItem( m_root, f.path() );
	if ( tvi )
		m_allDirs->setCurrentItem(tvi);
}

/**
@brief Clear and repopulate m_content with the entries found in m_file (setting the status right)
*/
void kdigestWidget::rereadEntries()
{
	// Check if called rightfully
	if ( ! m_file ) return;
	
	m_content->clear();
	
	DigestFile::DigestEntries es = m_file->entries();
	for ( DigestFile::DigestEntries::iterator it = es.begin(); it != es.end(); it++)
	{
		KListViewItem *lvi = new KListViewItem(
			m_content,
			m_content->lastItem(),
			(*it)->filename,
			(*it)->digest()
		);
		switch( (*it)->status )
		{
			case DigestFile::dsNotChecked:	// Do nothing
				break;
			case DigestFile::dsOk:		// Set the right icon and Digest
				lvi->setText(2, (*it)->actual());
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_ok", KIcon::Small));
				break;
			case DigestFile::dsBadDigest:	// Set the Digest...
				lvi->setText(2, (*it)->actual());
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			case DigestFile::dsNotFound:
				lvi->setText(2, "Not found");
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			case DigestFile::dsErrorAccess:
				lvi->setText(2, "Not accessible");
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			default:			// ..and then the wrong icon (for all other cases)
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
		}
	}
}

/**
@brief Slot called when the Check file action is selected
*/
void kdigestWidget::checkFile()
{
	if ( ! m_file ) return;

	connect( m_file, SIGNAL(cycleCompleted(int)), m_progress, SLOT(setProgress(int)) );
	m_progress->setTotalSteps( m_file->entries().count() );
	m_progress->setEnabled(true);
	
	m_allDirs->setEnabled(false); m_currDir->setEnabled(false);
	bool ok = m_file->testFile(m_allDirs->currentURL().path());
	m_allDirs->setEnabled(true); m_currDir->setEnabled(true);
	
	disconnect( m_file );
	m_progress->setEnabled(false);
	
	rereadEntries();
	
	if ( ok )
		m_statusLabel->setText(tr("Digested files are all present and matches the given digest"));
	else
		m_statusLabel->setText(tr("Some digested files 't present, or doesn't matches the given digest, see the window for details"));
}

/**
@brief Converts a foolish AFV file to SFV
*/
void kdigestWidget::convertAFV()
{
	if ( ! m_file ) return;
	if ( m_currDir->selectedItems()->count() > 1 )
	{
		cleanup();
		return;
	}

	QString filename = m_currDir->view()->currentFileItem()->url().path();
	filename = filename.left( filename.length() -3 ) + "sfv";
	
	m_file->saveDigest( filename );

	cleanup();
	
	m_currDir->del( *(m_currDir->selectedItems()), this, true, false );
}

/**
@brief Cleanups the digestfile related stuff
*/
inline void kdigestWidget::cleanup()
{
	m_content->clear();
	delete m_file;
	m_file = NULL;
}

/**
@brief Slot called when executed action 'create new digest'
*/
void kdigestWidget::createDigest()
{
	if ( m_currDir->selectedItems()->count() < 1 )
		return;
	
	// Ok now ask for the filename and format
	QString fn = KFileDialog::getSaveFileName(
		m_currDir->url().path(),
		"*.sfv|Simple File Verification\n"
		"*.md5 MD5SUM MD5SUMS|MD5 digest file\n",
		this,
		"Select name and type of digest"
		);
		
	if ( fn.isEmpty() )
		return;
	
	DigestFile *df = NULL;
	if ( fn.right(4).lower() == ".sfv" )
		df = new KDigest::SFVfile(this);
	else if (
		fn.right(4).lower() == ".md5" ||
		fn == "MD5SUM" ||
		fn == "MD5SUMS"
	   )
	   	df = new KDigest::MD5SUMfile(this);
	else
		return;
	
	KFileItemList fil(*m_currDir->selectedItems());
	
	for( KFileItemList::iterator it = fil.begin(); it != fil.end(); it++ )
		df->addFile( (*it)->url().path() );
	
	df->saveDigest( fn );
}

#include "kdigestwidget.moc"

