/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kdigest.h"
#include "sfvfile.h"

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>

static const char description[] =
    I18N_NOOP("SFV file checker");

static const char version[] = VERSION;

static KCmdLineOptions options[] =
{
	{ "s <file>", I18N_NOOP( "Check an SFV file" ), 0 },
	{ "m <file>", I18N_NOOP( "Check a MD5SUM file" ), 0 },
	{ "q", I18N_NOOP( "Suppresses all output" ), 0 },
	{ "c", I18N_NOOP( "Output at console instead of messagebox" ), 0 },
	{ "r", I18N_NOOP( "Display a final report of the checked files if fails" ), 0 },
	{ "n", I18N_NOOP( "Don't return check values (used for servicemenu)" ), 0},
	{ "+[filename]", I18N_NOOP( "Digest file to open" ), 0 },
	KCmdLineLastOption
};

int main(int argc, char **argv)
{
	KAboutData about("kdigest", I18N_NOOP("kdigest"), version, description,
		KAboutData::License_GPL, "(C) 2004 Diego 'Flameeyes' Petten", 0, 0, "dgp85@users.sourceforge.net");
	about.addAuthor( "Diego 'Flameeyes' Petten", 0, "dgp85@users.sourceforge.net" );
	KCmdLineArgs::init(argc, argv, &about);
	KCmdLineArgs::addCmdLineOptions( options );
	KApplication app;
	kdigest *mainWin = 0;

	if (app.isRestored())
	{
		RESTORE(kdigest);
	}
	else
	{
		// no session.. just start up normally
		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
		uint8_t flags = 0x00
			| ( ( args->isSet("q") ) ? KDigest::nicNoOutput : 0x00 )
			| ( ( args->isSet("c") ) ? KDigest::nicConsole : 0x00 )
			| ( ( args->isSet("r") ) ? KDigest::nicFinalReport : 0x00 );
		
		if ( ! args->getOption("s").isEmpty() && ! args->getOption("m").isEmpty() )
			qFatal("Please select only one digest to check.");
		
		if ( ! args->getOption("s").isEmpty() )
			if ( DigestFile::checkFile(KDigest::dtSFV, args->getOption("s"), flags) )
				return 0;
			else
				return args->isSet("nr") ? 0 : -1;
		if ( ! args->getOption("m").isEmpty() )
			if ( DigestFile::checkFile(KDigest::dtMD5SUM, args->getOption("m"), flags) )
				return 0;
			else
				return args->isSet("nr") ? 0 : -1;
		
		mainWin = new kdigest();
		app.setMainWidget( mainWin );
		mainWin->show();
	
		args->clear();
	}

	// mainWin has WDestructiveClose flag by default, so it will delete itself.
	return app.exec();
}
